@@ -5,7 +5,7 @@
  *
  *  Copyright (C) 2006-2014, ARM Limited, All Rights Reserved
  *
- *  This file is part of mbed TLS (https://polarssl.org)
+ *  This file is part of mbed TLS (https://tls.mbed.org)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 #define POLARSSL_ERR_RSA_BAD_INPUT_DATA                    -0x4080  /**< Bad input parameters to function. */
 #define POLARSSL_ERR_RSA_INVALID_PADDING                   -0x4100  /**< Input data contains invalid padding and is rejected. */
 #define POLARSSL_ERR_RSA_KEY_GEN_FAILED                    -0x4180  /**< Something failed during generation of a key. */
-#define POLARSSL_ERR_RSA_KEY_CHECK_FAILED                  -0x4200  /**< Key failed to pass the libraries validity check. */
+#define POLARSSL_ERR_RSA_KEY_CHECK_FAILED                  -0x4200  /**< Key failed to pass the library's validity check. */
 #define POLARSSL_ERR_RSA_PUBLIC_FAILED                     -0x4280  /**< The public key operation failed. */
 #define POLARSSL_ERR_RSA_PRIVATE_FAILED                    -0x4300  /**< The private key operation failed. */
 #define POLARSSL_ERR_RSA_VERIFY_FAILED                     -0x4380  /**< The PKCS#1 verification failed. */
@@ -199,6 +199,7 @@ int rsa_check_pub_priv( const rsa_context *pub, const rsa_context *prv );
 
 /**
  * \brief          Do an RSA public key operation
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      RSA context
  * \param input    input buffer
@@ -219,6 +220,7 @@ int rsa_public( rsa_context *ctx,
 
 /**
  * \brief          Do an RSA private key operation
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      RSA context
  * \param f_rng    RNG function (Needed for blinding)
@@ -241,6 +243,7 @@ int rsa_private( rsa_context *ctx,
  * \brief          Generic wrapper to perform a PKCS#1 encryption using the
  *                 mode from the context. Add the message padding, then do an
  *                 RSA operation.
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      RSA context
  * \param f_rng    RNG function (Needed for padding and PKCS#1 v2.1 encoding
@@ -265,6 +268,7 @@ int rsa_pkcs1_encrypt( rsa_context *ctx,
 
 /**
  * \brief          Perform a PKCS#1 v1.5 encryption (RSAES-PKCS1-v1_5-ENCRYPT)
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      RSA context
  * \param f_rng    RNG function (Needed for padding and RSA_PRIVATE)
@@ -288,6 +292,7 @@ int rsa_rsaes_pkcs1_v15_encrypt( rsa_context *ctx,
 
 /**
  * \brief          Perform a PKCS#1 v2.1 OAEP encryption (RSAES-OAEP-ENCRYPT)
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      RSA context
  * \param f_rng    RNG function (Needed for padding and PKCS#1 v2.1 encoding
@@ -318,6 +323,7 @@ int rsa_rsaes_oaep_encrypt( rsa_context *ctx,
  * \brief          Generic wrapper to perform a PKCS#1 decryption using the
  *                 mode from the context. Do an RSA operation, then remove
  *                 the message padding
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      RSA context
  * \param f_rng    RNG function (Only needed for RSA_PRIVATE)
@@ -330,9 +336,15 @@ int rsa_rsaes_oaep_encrypt( rsa_context *ctx,
  *
  * \return         0 if successful, or an POLARSSL_ERR_RSA_XXX error code
  *
- * \note           The output buffer must be as large as the size
- *                 of ctx->N (eg. 128 bytes if RSA-1024 is used) otherwise
- *                 an error is thrown.
+ * \note           The output buffer length \c output_max_len should be
+ *                 as large as the size ctx->len of ctx->N (eg. 128 bytes
+ *                 if RSA-1024 is used) to be able to hold an arbitrary
+ *                 decrypted message. If it is not large enough to hold
+ *                 the decryption of the particular ciphertext provided,
+ *                 the function will return POLARSSL_ERR_RSA_OUTPUT_TOO_LARGE.
+ *
+ * \note           The input buffer must be as large as the size
+ *                 of ctx->N (eg. 128 bytes if RSA-1024 is used).
  */
 int rsa_pkcs1_decrypt( rsa_context *ctx,
                        int (*f_rng)(void *, unsigned char *, size_t),
@@ -344,6 +356,7 @@ int rsa_pkcs1_decrypt( rsa_context *ctx,
 
 /**
  * \brief          Perform a PKCS#1 v1.5 decryption (RSAES-PKCS1-v1_5-DECRYPT)
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      RSA context
  * \param f_rng    RNG function (Only needed for RSA_PRIVATE)
@@ -356,9 +369,16 @@ int rsa_pkcs1_decrypt( rsa_context *ctx,
  *
  * \return         0 if successful, or an POLARSSL_ERR_RSA_XXX error code
  *
- * \note           The output buffer must be as large as the size
- *                 of ctx->N (eg. 128 bytes if RSA-1024 is used) otherwise
- *                 an error is thrown.
+ * \note           The output buffer length \c output_max_len should be
+ *                 as large as the size ctx->len of ctx->N (eg. 128 bytes
+ *                 if RSA-1024 is used) to be able to hold an arbitrary
+ *                 decrypted message. If it is not large enough to hold
+ *                 the decryption of the particular ciphertext provided,
+ *                 the function will return POLARSSL_ERR_RSA_OUTPUT_TOO_LARGE.
+ *
+ * \note           The input buffer must be as large as the size
+ *                 of ctx->N (eg. 128 bytes if RSA-1024 is used).
+ *
  */
 int rsa_rsaes_pkcs1_v15_decrypt( rsa_context *ctx,
                                  int (*f_rng)(void *, unsigned char *, size_t),
@@ -370,6 +390,7 @@ int rsa_rsaes_pkcs1_v15_decrypt( rsa_context *ctx,
 
 /**
  * \brief          Perform a PKCS#1 v2.1 OAEP decryption (RSAES-OAEP-DECRYPT)
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      RSA context
  * \param f_rng    RNG function (Only needed for RSA_PRIVATE)
@@ -384,9 +405,16 @@ int rsa_rsaes_pkcs1_v15_decrypt( rsa_context *ctx,
  *
  * \return         0 if successful, or an POLARSSL_ERR_RSA_XXX error code
  *
- * \note           The output buffer must be as large as the size
- *                 of ctx->N (eg. 128 bytes if RSA-1024 is used) otherwise
- *                 an error is thrown.
+ * \note           The output buffer length \c output_max_len should be
+ *                 as large as the size ctx->len of ctx->N (eg. 128 bytes
+ *                 if RSA-1024 is used) to be able to hold an arbitrary
+ *                 decrypted message. If it is not large enough to hold
+ *                 the decryption of the particular ciphertext provided,
+ *                 the function will return POLARSSL_ERR_RSA_OUTPUT_TOO_LARGE.
+ *
+ * \note           The input buffer must be as large as the size
+ *                 of ctx->N (eg. 128 bytes if RSA-1024 is used).
+ *
  */
 int rsa_rsaes_oaep_decrypt( rsa_context *ctx,
                             int (*f_rng)(void *, unsigned char *, size_t),
@@ -402,6 +430,7 @@ int rsa_rsaes_oaep_decrypt( rsa_context *ctx,
  * \brief          Generic wrapper to perform a PKCS#1 signature using the
  *                 mode from the context. Do a private RSA operation to sign
  *                 a message digest
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      RSA context
  * \param f_rng    RNG function (Needed for PKCS#1 v2.1 encoding and for
@@ -460,6 +489,7 @@ int rsa_rsassa_pkcs1_v15_sign( rsa_context *ctx,
 
 /**
  * \brief          Perform a PKCS#1 v2.1 PSS signature (RSASSA-PSS-SIGN)
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      RSA context
  * \param f_rng    RNG function (Needed for PKCS#1 v2.1 encoding and for
@@ -495,6 +525,7 @@ int rsa_rsassa_pss_sign( rsa_context *ctx,
  * \brief          Generic wrapper to perform a PKCS#1 verification using the
  *                 mode from the context. Do a public RSA operation and check
  *                 the message digest
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      points to an RSA public key
  * \param f_rng    RNG function (Only needed for RSA_PRIVATE)
@@ -525,6 +556,7 @@ int rsa_pkcs1_verify( rsa_context *ctx,
 
 /**
  * \brief          Perform a PKCS#1 v1.5 verification (RSASSA-PKCS1-v1_5-VERIFY)
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      points to an RSA public key
  * \param f_rng    RNG function (Only needed for RSA_PRIVATE)
@@ -553,6 +585,7 @@ int rsa_rsassa_pkcs1_v15_verify( rsa_context *ctx,
 /**
  * \brief          Perform a PKCS#1 v2.1 PSS verification (RSASSA-PSS-VERIFY)
  *                 (This is the "simple" version.)
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      points to an RSA public key
  * \param f_rng    RNG function (Only needed for RSA_PRIVATE)
@@ -587,6 +620,7 @@ int rsa_rsassa_pss_verify( rsa_context *ctx,
 /**
  * \brief          Perform a PKCS#1 v2.1 PSS verification (RSASSA-PSS-VERIFY)
  *                 (This is the version with "full" options.)
+ *                 (Thread-safe if POLARSSL_THREADING_C is enabled)
  *
  * \param ctx      points to an RSA public key
  * \param f_rng    RNG function (Only needed for RSA_PRIVATE)
