@@ -68,13 +68,10 @@
 #define strcasecmp _stricmp
 #endif
 
-#if defined(_MSC_VER) && !defined(inline)
-#define inline _inline
-#else
-#if defined(__ARMCC_VERSION) && !defined(inline)
+#if ( defined(__ARMCC_VERSION) || defined(_MSC_VER) ) && \
+    !defined(inline) && !defined(__cplusplus)
 #define inline __inline
-#endif /* __ARMCC_VERSION */
-#endif /*_MSC_VER */
+#endif
 
 /* Implementation that should never be optimized out by the compiler */
 static void polarssl_zeroize( void *v, size_t n ) {
@@ -1154,7 +1151,7 @@ static int ecp_randomize_jac( const ecp_group *grp, ecp_point *pt,
     /* Generate l such that 1 < l < p */
     do
     {
-        mpi_fill_random( &l, p_size, f_rng, p_rng );
+        MPI_CHK( mpi_fill_random( &l, p_size, f_rng, p_rng ) );
 
         while( mpi_cmp_mpi( &l, &grp->P ) >= 0 )
             MPI_CHK( mpi_shift_r( &l, 1 ) );
@@ -1539,7 +1536,7 @@ static int ecp_randomize_mxz( const ecp_group *grp, ecp_point *P,
     /* Generate l such that 1 < l < p */
     do
     {
-        mpi_fill_random( &l, p_size, f_rng, p_rng );
+        MPI_CHK( mpi_fill_random( &l, p_size, f_rng, p_rng ) );
 
         while( mpi_cmp_mpi( &l, &grp->P ) >= 0 )
             MPI_CHK( mpi_shift_r( &l, 1 ) );
@@ -1857,7 +1854,6 @@ int ecp_gen_keypair( ecp_group *grp, mpi *d, ecp_point *Q,
     {
         /* SEC1 3.2.1: Generate d such that 1 <= n < N */
         int count = 0;
-        unsigned char rnd[POLARSSL_ECP_MAX_BYTES];
 
         /*
          * Match the procedure given in RFC 6979 (deterministic ECDSA):
@@ -1868,8 +1864,7 @@ int ecp_gen_keypair( ecp_group *grp, mpi *d, ecp_point *Q,
          */
         do
         {
-            MPI_CHK( f_rng( p_rng, rnd, n_size ) );
-            MPI_CHK( mpi_read_binary( d, rnd, n_size ) );
+            MPI_CHK( mpi_fill_random( d, n_size, f_rng, p_rng ) );
             MPI_CHK( mpi_shift_r( d, 8 * n_size - grp->nbits ) );
 
             /*
