@@ -285,9 +285,6 @@ static const char *features[] = {
 #if defined(POLARSSL_SELF_TEST)
     "POLARSSL_SELF_TEST",
 #endif /* POLARSSL_SELF_TEST */
-#if defined(POLARSSL_SSL_AEAD_RANDOM_IV)
-    "POLARSSL_SSL_AEAD_RANDOM_IV",
-#endif /* POLARSSL_SSL_AEAD_RANDOM_IV */
 #if defined(POLARSSL_SSL_ALERT_MESSAGES)
     "POLARSSL_SSL_ALERT_MESSAGES",
 #endif /* POLARSSL_SSL_ALERT_MESSAGES */
@@ -345,6 +342,9 @@ static const char *features[] = {
 #if defined(POLARSSL_SSL_TRUNCATED_HMAC)
     "POLARSSL_SSL_TRUNCATED_HMAC",
 #endif /* POLARSSL_SSL_TRUNCATED_HMAC */
+#if defined(POLARSSL_SSL_ENABLE_MD5_SIGNATURES)
+    "POLARSSL_SSL_ENABLE_MD5_SIGNATURES",
+#endif /* POLARSSL_SSL_ENABLE_MD5_SIGNATURES */
 #if defined(POLARSSL_SSL_SET_CURVES)
     "POLARSSL_SSL_SET_CURVES",
 #endif /* POLARSSL_SSL_SET_CURVES */
@@ -363,6 +363,9 @@ static const char *features[] = {
 #if defined(POLARSSL_X509_ALLOW_UNSUPPORTED_CRITICAL_EXTENSION)
     "POLARSSL_X509_ALLOW_UNSUPPORTED_CRITICAL_EXTENSION",
 #endif /* POLARSSL_X509_ALLOW_UNSUPPORTED_CRITICAL_EXTENSION */
+#if defined(POLARSSL_X509_ALLOW_RELAXED_DATE)
+    "POLARSSL_X509_ALLOW_RELAXED_DATE",
+#endif /* POLARSSL_X509_ALLOW_RELAXED_DATE */
 #if defined(POLARSSL_X509_CHECK_KEY_USAGE)
     "POLARSSL_X509_CHECK_KEY_USAGE",
 #endif /* POLARSSL_X509_CHECK_KEY_USAGE */
