@@ -35,13 +35,15 @@
 
 #if !defined(POLARSSL_X509_CSR_WRITE_C) || !defined(POLARSSL_FS_IO) ||  \
     !defined(POLARSSL_PK_PARSE_C) || !defined(POLARSSL_SHA256_C) || \
-    !defined(POLARSSL_ENTROPY_C) || !defined(POLARSSL_CTR_DRBG_C)
+    !defined(POLARSSL_ENTROPY_C) || !defined(POLARSSL_CTR_DRBG_C) || \
+    !defined(POLARSSL_PEM_WRITE_C)
 int main( void )
 {
     polarssl_printf( "POLARSSL_X509_CSR_WRITE_C and/or POLARSSL_FS_IO and/or "
-            "POLARSSL_PK_PARSE_C and/or POLARSSL_SHA256_c and/or "
-            "POLARSSL_ENTROPY_C and/or POLARSSL_CTR_DRBG_C "
-            "not defined.\n");
+                     "POLARSSL_PK_PARSE_C and/or POLARSSL_SHA256_c and/or "
+                     "POLARSSL_ENTROPY_C and/or POLARSSL_CTR_DRBG_C and/or "
+                     "POLARSSL_PEM_WRITE_C "
+                     "not defined.\n");
     return( 0 );
 }
 #else
@@ -341,4 +343,4 @@ exit:
     return( ret );
 }
 #endif /* POLARSSL_X509_CSR_WRITE_C && POLARSSL_PK_PARSE_C && POLARSSL_FS_IO &&
-          POLARSSL_ENTROPY_C && POLARSSL_CTR_DRBG_C */
+          POLARSSL_ENTROPY_C && POLARSSL_CTR_DRBG_C && POLARSSL_PEM_WRITE_C */
